"use strict";
var BufferKeys, FlexBuffer, _writerBuilder, fn, j, k, len1, v;
FlexBuffer = (function() {
  FlexBuffer.SAFE_BUFFER_LENGTH = 10 * 1024 * 1024;
  FlexBuffer.GROW_FACTOR = 2;
  function FlexBuffer(arg, opts) {
    if (arg == null) {
      arg = 1024;
    }
    if (opts == null) {
      opts = {};
    }
    this._initArg = arg;
    this._initOpts = opts;
    this._resizeTime = 0;
    this._buildBuffer(arg, opts);
    this.GROW_FACTOR = opts.growFactor || FlexBuffer.GROW_FACTOR;
    this.SAFE_BUFFER_LENGTH = opts.safeBufferLength || FlexBuffer.SAFE_BUFFER_LENGTH;
    this;
  }
  FlexBuffer.prototype._buildBuffer = function(arg, opts) {
    if (opts == null) {
      opts = {};
    }
    if (typeof arg === 'number') {
      this._buffer = new Buffer(arg);
      return this._writeOffset = 0;
    } else if (Buffer.isBuffer(arg)) {
      this._buffer = arg;
      return this._writeOffset = arg.length;
    } else {
      this._buffer = new Buffer(arg, opts.encoding);
      return this._writeOffset = this._buffer.length;
    }
  };
  FlexBuffer.prototype._newBufferSize = function(delta) {
    var newSize;
    if (!this._buffer.length) {
      return delta;
    } else if (!this.SAFE_BUFFER_LENGTH) {
      return newSize = this._buffer.length * this.GROW_FACTOR;
    } else if (this._buffer.length < this.SAFE_BUFFER_LENGTH && this.GROW_FACTOR !== 0) {
      newSize = this._buffer.length * this.GROW_FACTOR;
      if (newSize - this._buffer.length > delta) {
        return newSize;
      } else {
        return this._buffer.length + delta;
      }
    } else {
      return this._writeOffset + delta;
    }
  };
  FlexBuffer.prototype._resize = function(delta) {
    var oldBuffer;
    oldBuffer = this._buffer;
    this._buffer = new Buffer(this._newBufferSize(delta));
    oldBuffer.copy(this._buffer, 0);
    oldBuffer = null;
    return this._resizeTime += 1;
  };
  FlexBuffer.prototype._resizeIfRequired = function(delta) {
    if (this._writeOffset + delta > this._buffer.length) {
      return this._resize(delta);
    }
  };
  FlexBuffer.prototype._writeByte = function(byteNum) {
    return this._buffer[this._writeOffset++] = byteNum;
  };
  FlexBuffer.prototype._writeNumber = function(num) {
    this._resizeIfRequired(1);
    this._writeByte(num);
    return 1;
  };
  FlexBuffer.prototype._writeBuffer = function(buf) {
    var len;
    len = buf.length;
    this._resizeIfRequired(len);
    buf.copy(this._buffer, this._writeOffset);
    this._writeOffset += len;
    return len;
  };
  FlexBuffer.prototype._writeArray = function(arr) {
    var i, j, len, len1;
    len = arr.length;
    this._resizeIfRequired(len);
    for (j = 0, len1 = arr.length; j < len1; j++) {
      i = arr[j];
      this._writeByte(i);
    }
    return len;
  };
  FlexBuffer.prototype._writeString = function(str, encoding) {
    var len;
    len = Buffer.byteLength(str, encoding);
    this._resizeIfRequired(len);
    this._buffer.write(str, this._writeOffset, len, encoding);
    this._writeOffset += len;
    return len;
  };
  FlexBuffer.prototype.write = function(value, encoding) {
    if (typeof value === 'number') {
      return this._writeNumber(value);
    } else if (Buffer.isBuffer(value)) {
      return this._writeBuffer(value);
    } else if (Array.isArray(value)) {
      return this._writeArray(value);
    } else if (typeof value === 'string') {
      return this._writeString(value, encoding);
    } else {
      throw TypeError("Unvalid data type");
    }
  };
  FlexBuffer.prototype.slice = function(start, end, newBuffer) {
    var _buf;
    if (start == null) {
      start = 0;
    }
    if (end == null) {
      end = this.length;
    }
    if (newBuffer == null) {
      newBuffer = false;
    }
    _buf = this._buffer.slice(start, end);
    if (newBuffer) {
      return new Buffer(_buf);
    } else {
      return _buf;
    }
  };
  FlexBuffer.prototype.toBuffer = function(newBuffer) {
    if (newBuffer == null) {
      newBuffer = false;
    }
    return this.slice(null, null, newBuffer);
  };
  FlexBuffer.prototype.toString = function() {
    return this.toBuffer().toString();
  };
  FlexBuffer.prototype.inspect = function() {
    return "[FlexBuffer]\n    length: " + this.length + ",\n    bufferLength: " + this.bufferLength + "\n    _resizeTime: " + this._resizeTime + "\n    _buffer: " + (this._buffer.inspect());
  };
  FlexBuffer.prototype.reset = function() {
    this._resizeTime = 0;
    this.release();
    return this._buildBuffer(this._initArg, this._initOpts);
  };
  FlexBuffer.prototype.release = function() {
    this._buffer = null;
    return this.flush();
  };
  FlexBuffer.prototype.flush = function() {
    return this._writeOffset = 0;
  };
  return FlexBuffer;
})();
Object.defineProperties(FlexBuffer.prototype, {
  'bufferLength': {
    get: function() {
      if (this._buffer) {
        return this._buffer.length;
      } else {
        return 0;
      }
    }
  },
  'length': {
    get: function() {
      return this._writeOffset;
    }
  }
});
_writerBuilder = function(len, k, v) {
  return FlexBuffer.prototype[k] = function(val) {
    this._resizeIfRequired(len);
    v.call(this._buffer, val, this._writeOffset, false);
    this._writeOffset += len;
    return len;
  };
};
BufferKeys = Object.keys(Buffer.prototype);
Object.defineProperties(FlexBuffer.prototype, {
  'parent': {
    enumerable: true,
    get: function() {
      return this.toBuffer(false).parent;
    }
  },
  'offset': {
    enumerable: true,
    get: function() {
      return this.toBuffer(false).offset;
    }
  }
});
fn = function(k, v) {
  var arr;
  if (k.indexOf('write') === 0) {
    arr = k.match(/\d+/);
    if (arr && arr[0]) {
      return _writerBuilder.call(this, arr[0] / 8, k, v);
    } else if (/Double/.test(k)) {
      return _writerBuilder.call(this, 8, k, v);
    } else if (/Float/.test(k)) {
      return _writerBuilder.call(this, 4, k, v);
    } else {
      return FlexBuffer.prototype[k] = function(val, byteLength) {
        this._resizeIfRequired(byteLength);
        v.call(this._buffer, val, this._writeOffset, byteLength, false);
        this._writeOffset += byteLength;
        return byteLength;
      };
    }
  } else {
    return FlexBuffer.prototype[k] = function() {
      var dataBuf;
      dataBuf = this.toBuffer(false);
      return v.apply(dataBuf, arguments);
    };
  }
};
for (j = 0, len1 = BufferKeys.length; j < len1; j++) {
  k = BufferKeys[j];
  if (k === 'parent' || k === 'offset') {
    continue;
  }
  v = Buffer.prototype[k];
  if ((FlexBuffer.prototype[k] != null) || typeof v !== 'function') {
    continue;
  }
  fn(k, v);
}
module.exports = FlexBuffer;
